local World = require('classic'):extend()

local loader        = require('world.world_loader')
local lighting      = require('world.world_lighting')
local LightSource = require("utils.LightSource")

local ffi         = require("ffi")
local HMath       = ffi.load("HMath")
ffi.cdef [[
    typedef struct
    {
        float x;
        float y;
    } HVector2f;
    
    HVector2f toScreeni(int x, int y, int xorigin, int yorigin, int tile_width, int tile_height);
]]

function World:new(tilemap, tilesheet)
    self.width = 0
    self.height = 0

    self.light_sources = {}
    self.world = {}

    self.tilehseet = tilesheet
    self.tilemap = {}

    loader.loadTileMap(self, tilemap)
    loader.loadTileSheet(self, tilesheet)

    lighting.addLighting(self, LightSource(2, 2, nil, 4))
end

function World:draw()
    for y = 1, self.height do
		for x = 1, self.width do
			local tile_index = hmath.getIndexFromXY(x - 1, y - 1, self.width)
			local tile = self.world[tile_index]

			if not self.tilemap[tile.tileset] then
				goto continue
			end

			light_color = tile.light_level

			love.graphics.setColor(light_color:toNumbers())

            local v = HMath.toScreeni(x, y, 5, 1, TILE_WIDTH, TILE_HEIGHT)
            if y == 1 and x == 1 then
                print(v.x, v.y)
            end
			love.graphics.draw(self.tilesheet, self.tilemap[tile.tileset], v.x, v.y)

			::continue::
		end
	end
end

function World:update()
    
end

return World