local Color         = require("utils.Color")
local LightSource   = require("utils.LightSource")

local lighting = {}

local function spreadLight(self, x, y, radius, intensity, color, queue)
    local index = hmath.getIndexFromXY(x, y, self.width)

    if
        (intensity < 1
        or queue[index] and intensity ~= radius)
        or index < 0 or x < 0 or y < 0
        or x > self.width - 1 or y >= self.height or index >= self.size
    then
        return false
    end

    local color_mul = hmath.normalize(intensity, 0, radius)    
    local fcolor = self.world[index].light_level + (color * color_mul)

    if fcolor:getBrightness() < LightSource.ambient:getBrightness() then
        fcolor = Color(LightSource.ambient)
    end

    self.world[index].light_level = fcolor
    -- self.world[index].color = Color(1, 1, 1, 1)
    
    queue[index] = true

    return true
end

local function spreadLightRight(self, x, y, radius, intensity, color, queue)
    if spreadLight(self, x, y, radius, intensity, color, queue) then
        spreadLightRight(self, x + 1, y, radius, intensity - 1, color, queue)
        spreadLightRight(self, x, y - 1, radius, intensity - 1, color, queue)
        spreadLightRight(self, x, y + 1, radius, intensity - 1, color, queue)
    end
end

local function spreadLightLeft(self, x, y, radius, intensity, color, queue)
    if spreadLight(self, x, y, radius, intensity, color, queue) then
        spreadLightLeft(self, x - 1, y, radius, intensity - 1, color, queue)
        spreadLightLeft(self, x, y - 1, radius, intensity - 1, color, queue)
        spreadLightLeft(self, x, y + 1, radius, intensity - 1, color, queue)
    end
end

-- public functions
function lighting.calculateWorldLighting(self, light_sources)
    local lit_queue

    for _, v in ipairs(light_sources) do
        lit_queue = {}

        spreadLightLeft(self, v.x, v.y, v.radius, v.radius, v.color, lit_queue)
        spreadLightRight(self, v.x, v.y, v.radius, v.radius, v.color, lit_queue)
    end
end

function lighting.addLighting(self, ligh_source)
    table.insert(self.light_sources, ligh_source)

    lighting.calculateWorldLighting(self, {ligh_source})
end

return lighting